--[[--
@module MyDungeonsBook
]]

--[[--
Utils
@section Utils
]]

local castStartStopEvents = "UNIT_SPELLCAST_CHANNEL_START UNIT_SPELLCAST_CHANNEL_STOP UNIT_SPELLCAST_CHANNEL_UPDATE UNIT_SPELLCAST_DELAYED UNIT_SPELLCAST_FAILED UNIT_SPELLCAST_FAILED_QUIET UNIT_SPELLCAST_INTERRUPTED UNIT_SPELLCAST_START UNIT_SPELLCAST_STOP UNIT_SPELLCAST_SUCCEEDED";

local customTrigger = [[function(allstates, event, sourceUnit, castId, spellId)
    if (not sourceUnit) then
        return;
    end
    if (not castId) then
        castId = string.format("%s-%s", spellId, UnitGUID(sourceUnit));
    end
    if (sourceUnit and UnitIsEnemy(sourceUnit, "player")) then
        if (event == "UNIT_SPELLCAST_START"
            or event == "UNIT_SPELLCAST_DELAYED"
            or event == "UNIT_SPELLCAST_CHANNEL_START"
            or event == "UNIT_SPELLCAST_CHANNEL_UPDATE") then
            if (spellId == SPELL_ID) then
                local name, _, icon, startMs, endMs = UnitCastingInfo(sourceUnit);
                if (not name) then
                    name, _, icon, startMs, endMs = UnitChannelInfo(sourceUnit);
                end
                local duration = (endMs - startMs) / 1000;
                local expiration = endMs / 1000;
                allstates[castId] = {
                    show = true,
                    changed = true,
                    autoHide = true,
                    castId = castId,
                    icon = icon,
                    duration = duration,
                    expirationTime = expiration,
                    showIcon = true,
                    name = name,
                    progressType = "timed"
                };
                return true;
            end
        end

        if (event == "UNIT_SPELLCAST_STOP"
            or event == "UNIT_SPELLCAST_SUCCEEDED"
            or event == "UNIT_SPELLCAST_INTERRUPTED"
            or event == "UNIT_SPELLCAST_FAILED"
            or event == "UNIT_SPELLCAST_FAILED_QUIET"
            or event == "UNIT_SPELLCAST_CHANNEL_STOP") then
            if (spellId == SPELL_ID) then
                allstates[castId] = {
                    show = false,
                    changed = true,
                    autoHide = true,
                    castId = castId
                };
                return true;
            end

        end
    end
end]];

function MyDungeonsBook:WA_Interrupt_Template_Get()
    return {
        ["iconSource"] = -1,
        ["xOffset"] = 0,
        ["yOffset"] = 0,
        ["anchorPoint"] = "CENTER",
        ["cooldownSwipe"] = true,
        ["cooldownEdge"] = false,
        ["icon"] = true,
        ["triggers"] = {
            [1] = {
                ["trigger"] = {
                    ["spellId"] = "",
                    ["duration"] = "",
                    ["names"] = {
                    },
                    ["use_sourceFlags"] = false,
                    ["use_cloneId"] = true,
                    ["debuffType"] = "HELPFUL",
                    ["type"] = "custom",
                    ["custom_hide"] = "timed",
                    ["custom_type"] = "stateupdate",
                    ["subeventPrefix"] = "SPELL",
                    ["use_sourceFlags2"] = true,
                    ["event"] = "Combat Log",
                    ["sourceFlags2"] = "Hostile",
                    ["events"] = castStartStopEvents,
                    ["use_spellId"] = true,
                    ["spellIds"] = {
                    },
                    ["use_sourceUnit"] = false,
                    ["check"] = "event",
                    ["unit"] = "player",
                    ["custom"] = customTrigger,
                    ["subeventSuffix"] = "_CAST_START",
                },
                ["untrigger"] = {
                },
            },
            ["activeTriggerMode"] = -10,
        },
        ["internalVersion"] = 51,
        ["keepAspectRatio"] = false,
        ["selfPoint"] = "CENTER",
        ["desaturate"] = false,
        ["subRegions"] = {
            [1] = {
                ["type"] = "subbackground",
            },
            [2] = {
                ["text_shadowXOffset"] = 0,
                ["text_text_format_s_format"] = "none",
                ["text_text"] = "Kick!",
                ["text_shadowColor"] = {
                    [1] = 0,
                    [2] = 0,
                    [3] = 0,
                    [4] = 1,
                },
                ["text_selfPoint"] = "AUTO",
                ["text_automaticWidth"] = "Auto",
                ["text_fixedWidth"] = 64,
                ["anchorYOffset"] = 0,
                ["text_justify"] = "CENTER",
                ["rotateText"] = "NONE",
                ["type"] = "subtext",
                ["text_color"] = {
                    [1] = 1,
                    [2] = 1,
                    [3] = 1,
                    [4] = 1,
                },
                ["text_font"] = "Friz Quadrata TT",
                ["text_shadowYOffset"] = 0,
                ["text_wordWrap"] = "WordWrap",
                ["text_visible"] = true,
                ["text_anchorPoint"] = "OUTER_BOTTOM",
                ["text_fontSize"] = 32,
                ["anchorXOffset"] = 0,
                ["text_fontType"] = "OUTLINE",
            },
            [3] = {
                ["glowFrequency"] = 0.25,
                ["type"] = "subglow",
                ["glowXOffset"] = 0,
                ["glowType"] = "Pixel",
                ["glowLength"] = 10,
                ["glowYOffset"] = 0,
                ["glowColor"] = {
                    [1] = 0,
                    [2] = 1,
                    [3] = 0.59607843137255,
                    [4] = 1,
                },
                ["useGlowColor"] = true,
                ["glow"] = true,
                ["glowScale"] = 1,
                ["glowThickness"] = 3,
                ["glowLines"] = 8,
                ["glowBorder"] = false,
            },
        },
        ["height"] = 128,
        ["load"] = {
            ["size"] = {
                ["multi"] = {
                },
            },
        },
        ["authorMode"] = true,
        ["regionType"] = "icon",
        ["authorOptions"] = {
        },
        ["cooldownTextDisabled"] = false,
        ["cooldown"] = true,
        ["animation"] = {
            ["start"] = {
                ["type"] = "none",
                ["easeStrength"] = 3,
                ["duration_type"] = "seconds",
                ["easeType"] = "none",
            },
            ["main"] = {
                ["type"] = "none",
                ["easeStrength"] = 3,
                ["duration_type"] = "seconds",
                ["easeType"] = "none",
            },
            ["finish"] = {
                ["type"] = "none",
                ["easeStrength"] = 3,
                ["duration_type"] = "seconds",
                ["easeType"] = "none",
            },
        },
        ["zoom"] = 0.3,
        ["color"] = {
            [1] = 1,
            [2] = 1,
            [3] = 1,
            [4] = 1,
        },
        ["frameStrata"] = 1,
        ["id"] = "",
        ["config"] = {
        },
        ["alpha"] = 1,
        ["width"] = 128,
        ["anchorFrameType"] = "SCREEN",
        ["uid"] = "",
        ["inverse"] = false,
        ["actions"] = {
            ["start"] = {
                ["sound"] = "Interface\\Addons\\WeakAuras\\PowerAurasMedia\\Sounds\\PUNCH.ogg",
                ["do_sound"] = true,
            },
            ["init"] = {
            },
            ["finish"] = {
            },
        },
        ["conditions"] = {
        },
        ["information"] = {
        },
        ["parent"] = "",
    };
end

--[[--
@param[type=table] auraData
@param[type=number|nil] instanceId
@param[type=number|nil] bossId
@param[type=number|nil] affixId
@return[type=boolean]
]]
function MyDungeonsBook:WA_Interrupt_Aura_Create(auraData, instanceId, bossId, affixId)
    local wa = self:WA_Interrupt_Template_Get();
    local spellId = auraData.spellId;
    local _, _, icon, castTime = GetSpellInfo(spellId);
    wa.triggers[1].trigger.spellId = string.format("%s", spellId);
    wa.triggers[1].trigger.duration = string.format("%s", castTime / 1000);
    wa.triggers[1].trigger.custom = string.gsub(wa.triggers[1].trigger.custom, "SPELL_ID", spellId);
    wa.displayIcon = icon;
    wa.id = self:Wa_GenerateId(auraData, instanceId, bossId, affixId, "Interrupt");

    return self:Wa_AddNewWeakAura(wa, auraData, instanceId, bossId, affixId);
end

function MyDungeonsBook:Wa_ContextMenuItem_Interrupt_Create(spellId, instanceId, bosses, affixes)
    local menuItemsForBosses = {};
    for bossId, bossInfo in pairs(bosses) do
        tinsert(menuItemsForBosses, {
            text = bossInfo.name,
            func = function()
                self:WA_Interrupt_Aura_Create({spellId = spellId}, instanceId, bossId);
            end
        });
    end
    local menuItemsForAffixes = {};
    for _, affixId in pairs(affixes) do
        tinsert(menuItemsForAffixes, {
            text = self:GetAffixMetaInfo(affixId).name,
            func = function()
                self:WA_Interrupt_Aura_Create({ spellId = spellId }, nil, nil, affixId);
            end
        });
    end
    return {
        text = "Interrupt",
        hasArrow = true,
        menuList = {
            {
                text = "Trash",
                func = function()
                    self:WA_Interrupt_Aura_Create({ spellId = spellId }, instanceId, nil, nil);
                end
            },
            {
                text = "Boss",
                hasArrow = true,
                menuList = menuItemsForBosses
            },
            {
                text = "Affix",
                hasArrow = true,
                menuList = menuItemsForAffixes
            }
        }
    };
end
